import { h as hash, a as arrEquals, S as Store, f as format, i as intlHelpers, c as createGetTranslator, b as calcLocales, r as resolveProvidedArgs, t as translate, d as castArray } from "./index2.js";
import { m } from "./index2.js";
import { jsx, Fragment as Fragment$1 } from "react/jsx-runtime";
import { createContext, useMemo, useState, useEffect, useRef, useContext, Fragment } from "react";
import "@formatjs/intl-localematcher";
import "@formatjs/icu-messageformat-parser";
import "intl-messageformat";
const TranslationContext = createContext({
  locale: typeof navigator !== "undefined" ? navigator.language : void 0
});
const TranslationContextProvider = ({ locale, children }) => {
  const value = useMemo(() => ({ locale }), [locale]);
  return /* @__PURE__ */ jsx(TranslationContext.Provider, { value, children });
};
function useStore(store, ...locales) {
  const [, setCounter] = useState(0);
  useEffect(() => {
    return store.subscribe(locales, () => {
      setCounter((c) => c + 1);
    });
  }, [store, hash(locales)]);
  const dicts = store.getAll(...locales);
  const ref = useRef(dicts);
  if (ref.current !== dicts && !arrEquals(ref.current, dicts)) {
    ref.current = dicts;
  }
  return ref.current;
}
function createTranslator(options) {
  const store = new Store(options);
  const {
    sourceLocale,
    fallbackLocale,
    fallbackToLessSpecific = true,
    fallback: defaultFallback,
    fallbackIgnoresFallbackLocales = false,
    placeholder: defaultPlaceholder,
    warn,
    dateTimeFormatOptions,
    listFormatOptions,
    numberFormatOptions,
    pluralRulesOptions,
    relativeTimeFormatOptions,
    ignoreMissingArgs,
    provideArgs
  } = options;
  function useProvidedArgs() {
    const [args, setArgs] = useState(resolveProvidedArgs(provideArgs));
    useEffect(() => {
      const handles = [];
      for (const [, value] of Object.entries(provideArgs ?? {})) {
        if (typeof value === "object" && value !== null && "subscribe" in value) {
          const handle = value.subscribe(() => {
            setArgs(resolveProvidedArgs(provideArgs));
          });
          handles.push(handle);
        }
      }
      return () => {
        for (const handle of handles) {
          handle();
        }
      };
    }, []);
    return args;
  }
  const useTranslator = (overrideLocale) => {
    const contextLocale = useContext(TranslationContext).locale;
    const locale = overrideLocale ?? contextLocale ?? sourceLocale;
    const dicts = useStore(store, locale, ...calcLocales(locale, fallbackToLessSpecific, fallbackLocale));
    const [sourceDict] = useStore(store, sourceLocale);
    const providedArgs = useProvidedArgs();
    return useMemo(() => {
      const t2 = (id, ...[values, options2]) => {
        const fallback = (options2 == null ? void 0 : options2.fallback) ?? defaultFallback;
        const placeholder = (options2 == null ? void 0 : options2.placeholder) ?? defaultPlaceholder;
        return translate({
          dicts,
          sourceDict,
          id,
          values,
          fallback,
          fallbackIgnoresFallbackLocales,
          placeholder,
          locale,
          warn,
          cache: store.cache,
          ignoreMissingArgs,
          providedArgs
        });
      };
      return Object.assign(
        t2,
        {
          locale,
          unknown: t2,
          dynamic: t2,
          format(template, ...[values]) {
            return format({
              template,
              values,
              locale,
              cache: store.cache,
              ignoreMissingArgs,
              providedArgs
            });
          },
          ...intlHelpers({
            cache: store.cache,
            transform: (fn) => fn(locale),
            dateTimeFormatOptions,
            listFormatOptions
          })
        }
      );
    }, [locale, dicts, sourceDict]);
  };
  function TranslatorComponent({
    id,
    values,
    options: options2
  }) {
    const contextLocale = useContext(TranslationContext).locale;
    const locale = (options2 == null ? void 0 : options2.locale) ?? contextLocale ?? sourceLocale;
    const dicts = useStore(store, locale, ...calcLocales(locale, fallbackToLessSpecific, fallbackLocale));
    const [sourceDict] = useStore(store, sourceLocale);
    const providedArgs = useProvidedArgs();
    const fallback = (options2 == null ? void 0 : options2.fallback) ?? defaultFallback;
    const placeholder = (options2 == null ? void 0 : options2.placeholder) ?? defaultPlaceholder;
    const text = useMemo(
      () => translate({
        dicts,
        sourceDict,
        id,
        values,
        fallback,
        fallbackIgnoresFallbackLocales,
        placeholder,
        locale,
        warn,
        cache: store.cache,
        ignoreMissingArgs,
        providedArgs
      }),
      [locale, dicts, sourceDict, providedArgs, id, values, fallback, placeholder]
    );
    const textArray = castArray(text);
    const Component = (options2 == null ? void 0 : options2.component) ?? Fragment;
    return /* @__PURE__ */ jsx(Fragment$1, { children: textArray.map((line, index) => /* @__PURE__ */ jsx(Component, { children: line }, index)) });
  }
  const createTranslatorComponent = (id, ...[values, options2]) => {
    return /* @__PURE__ */ jsx(TranslatorComponent, { id, values, options: options2 });
  };
  const RenderComponent = ({
    renderFn,
    dependecies = [renderFn]
  }) => {
    const t2 = useTranslator();
    const providedArgs = useProvidedArgs();
    const value = useMemo(
      () => renderFn(t2, {
        providedArgs
      }),
      [t2, providedArgs, ...dependecies]
    );
    return /* @__PURE__ */ jsx(Fragment$1, { children: value });
  };
  const render = (renderFn, dependecies) => {
    return /* @__PURE__ */ jsx(RenderComponent, { renderFn, dependecies });
  };
  const t = Object.assign(createTranslatorComponent, {
    locale: render((t2) => t2.locale, []),
    unknown: createTranslatorComponent,
    dynamic: createTranslatorComponent,
    format(template, ...[values]) {
      return render(
        (t2, { providedArgs }) => format({
          template,
          values,
          locale: t2.locale,
          cache: store.cache,
          ignoreMissingArgs,
          providedArgs
        }),
        [template, hash(values)]
      );
    },
    render,
    ...intlHelpers({
      cache: store.cache,
      transform: (fn) => render((t2) => fn(t2.locale)),
      dateTimeFormatOptions,
      listFormatOptions,
      numberFormatOptions,
      pluralRulesOptions,
      relativeTimeFormatOptions
    })
  });
  return {
    getTranslator: createGetTranslator(store, options),
    useTranslator,
    t,
    clearDicts() {
      store.clear();
    }
  };
}
export {
  TranslationContext,
  TranslationContextProvider,
  createTranslator,
  m as mergeDicts
};
//# sourceMappingURL=react.js.map
